package dev.fitko.fitconnect.api.config;

import dev.fitko.fitconnect.api.config.defaults.ResourcePaths;
import dev.fitko.fitconnect.api.config.defaults.SchemaConfig;
import dev.fitko.fitconnect.api.exceptions.client.FitConnectInitialisationException;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.With;
import lombok.experimental.Accessors;

import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

@Getter
@Builder
@Accessors(chain = true)
@NoArgsConstructor
@AllArgsConstructor
public class ApplicationConfig {

    @Builder.Default
    public static final String AUTH_TAG_SPLIT_TOKEN = "\\.";

    @Builder.Default
    private String httpProxyHost = "";

    @Builder.Default
    private Integer httpProxyPort = 0;

    @Builder.Default
    private Integer requestTimeoutInSeconds = 30;

    @Builder.Default
    private URI setSchemaWriteVersion = SchemaConfig.SET_V_1_0_1.getSchemaUri();

    @Builder.Default
    private URI metadataSchemaWriteVersion = SchemaConfig.METADATA_V_1_0_0.getSchemaUri();

    @Builder.Default
    private URI destinationSchemaVersion = SchemaConfig.XZUFI_DESTINATION_SCHEMA.getSchemaUri();

    private String submissionDataSchemaPath;

    private SenderConfig senderConfig;

    private SubscriberConfig subscriberConfig;

    @With
    @Builder.Default
    private Map<EnvironmentName, Environment> environments = new HashMap<>();

    private EnvironmentName activeEnvironment;

    public boolean isAutoRejectEnabled() {
        return getCurrentEnvironment().getEnableAutoReject();
    }

    public boolean isSkipSubmissionDataValidation() {
        return getCurrentEnvironment().getSkipSubmissionDataValidation();
    }

    public boolean isAllowInsecurePublicKey() {
        return getCurrentEnvironment().getAllowInsecurePublicKey();
    }

    public Environment getCurrentEnvironment() {
        if (environments.containsKey(activeEnvironment)) {
            return environments.get(activeEnvironment);
        } else {
            throw new FitConnectInitialisationException("No environment with name '" + activeEnvironment.getName() + "' found. Available environments are: " + getAvailableEnvironmentNames());
        }
    }

    public String getOAuthTokenEndpoint() {
        return getAuthBaseUrl() + ResourcePaths.AUTH_TOKEN_PATH;
    }

    public String getSubmissionsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSIONS_PATH;
    }

    public String getSubmissionEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSION_PATH;
    }

    public String getAttachmentEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.SUBMISSION_ATTACHMENT_PATH;
    }

    public String getEventsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.EVENTS_PATH;
    }

    public String getDestinationsEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.DESTINATIONS_PATH;
    }

    public String getDestinationsKeyEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.DESTINATIONS_KEY_PATH;
    }

    public String getSelfServicePortalWellKnownKeysEndpoint() {
        return getSelfServicePortalBaseUrl() + ResourcePaths.WELL_KNOWN_KEYS_PATH;
    }

    public String getSubmissionServiceWellKnownKeysEndpoint() {
        return getSubmissionBaseUrl() + ResourcePaths.WELL_KNOWN_KEYS_PATH;
    }

    public String getAreaEndpoint() {
        return getRoutingBaseUrl() + ResourcePaths.ROUTING_AREA_PATH;
    }

    public String getRoutesEndpoint() {
        return getRoutingBaseUrl() + ResourcePaths.ROUTING_ROUTE_PATH;
    }

    public String getWellKnownKeysPath() {
        return ResourcePaths.WELL_KNOWN_KEYS_PATH;
    }

    private String getSubmissionBaseUrl() {
        return getCurrentEnvironment().getSubmissionBaseUrls()
                .stream()
                .findFirst()
                .orElseThrow(() -> new FitConnectInitialisationException("No submission base url found, expected at least one"));
    }

    private String getSelfServicePortalBaseUrl() {
        return getCurrentEnvironment().getSelfServicePortalBaseUrl();
    }

    private String getRoutingBaseUrl() {
        return getCurrentEnvironment().getRoutingBaseUrl();
    }

    private String getAuthBaseUrl() {
        return getCurrentEnvironment().getAuthBaseUrl();
    }

    private String getAvailableEnvironmentNames() {
        return environments.keySet()
                .stream()
                .map(EnvironmentName::getName)
                .collect(Collectors.joining(" | "));
    }

    public boolean isProxySet() {
        return (httpProxyPort != null && httpProxyPort > 0) && (httpProxyHost != null && !httpProxyHost.isEmpty());
    }
}

